;******************************************************************************
;   How to use this file:                                                     *
;   ======================                                                    *
;                                                                             *
;   This file provides a basic template for writing dsPIC30F trap handlers    *
;   in ASM30 assembler language.                                              *
;                                                                             *
;   Add this file into your MPLAB project. Build your project, program        *
;   the device and run. If any trap occurs during code execution, the         *
;   processor vectors to one of these routines.                               *
;                                                                             *
;   For additional information about dsPIC 30F  architecture and language     *
;   tools, refer to the following documents:                                  *
;                                                                             *
;   MPLAB C30 Compiler User's Guide                       : DS51284           *
;   dsPIC 30F Assembler, Linker and Utilities User's Guide: DS51317           *
;   dsPIC 30F 16-bit MCU Family Reference Manual          : DS70046           *
;   dsPIC 30F Power and Motion Control Family Data Sheet  : DS70082           *
;   dsPIC 30F Programmer's Reference Manual               : DS70030           *
;                                                                             *
;******************************************************************************
;                                                                             *
;    Author              :                                                    *
;    Company             :                                                    *
;    Filename            :  traps.s                                           *
;    Date                :  09/12/2004                                        *
;    File Version        :  1.00                                              *
;                                                                             *
;    Devices Supported by this file:                                          *
;                        All dsPIC30F devices                                 *
;                                                                             *
;******************************************************************************



        .include "p30f4013.inc."


        .global  __OscillatorFail
        .global  __AddressError
        .global  __StackError
        .global  __MathError

        .global  __AltOscillatorFail
        .global  __AltAddressError
        .global  __AltStackError
        .global  __AltMathError
		.global  __reset               ;; << NEED THIS !!!!!

;------------------------------------------------------------------------------
		;; Configuration bits
		;; Derived from p30f4013.inc file
		config __FOSC, CSW_FSCM_ON & EC
		/* Configure FOSC Register as the following value
		   0x3FFF & 0xFBEB = 0x3BEB 
		   FCKSM<1:0> Clock Switching and Monitor Selection Configuration bits
			00 = Clock switching is enabled, Fail-Safe Clock 
							Monitor is enabled
		   FOS<2:0>: Oscillator Group Selection on POR bits
			011 = EXT: External Oscillator; OSC1/OSC2 pins; 
			External Oscillator configuration selected by FPR<4:0> bits
		   FPR<4:0>: Oscillator Selection within Primary Group bits.
			01011 = In the EC mode, the OSC1 pin can be driven by CMOS drivers. 
			In this mode, the OSC1 pin is high-impedance and the OSC2 pin 
			is the clock output (FOSC/4).
		   See Section 20.0 of data sheet for more information
			Section 7.8 of dsPIC30F Family Reference Sheet
		   Clock input on OSC1 is 32.768 MHz
			*/
		config __FWDT, WDT_OFF
		config __FBORPOR, PBOR_OFF & BORV_27 & PWRT_16 & MCLR_EN
		config __FGS, CODE_PROT_OFF
		
		;; Variable Name Definitions
		;; RAM variables
		.bss
		DelayCount:.space 2	;; Allocating space (in bytes) to variable
		DelayCount2:.space 2
		DelayCount3:.space 2
		DelayCount4:.space 2	
	


		data: .space 1
/*
		; dsPIC registers
	 	.equ CORCONL, CORCON
	 	.equ PSV,2
		
		.section .const,psv
Hello:
		.ascii "Hello World!"
*/



         .text	;; begin the progam code here, start of executiable code

;; MACRO LISTING ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
SetA0:	.macro
	;; RF2 =  A0
		bset LATF, #2
	.endm
ClrA0:	.macro
		bclr LATF, #2
	.endm

SetCS:	.macro
	;; RF3 = !CS
		bset LATF, #3
	.endm
ClrCS:	.macro
		bclr LATF, #3
	.endm

SetRD:	.macro
	;; RF0 = !RD
		bset LATF, #0
	.endm
ClrRD:	.macro
		bclr LATF, #0
	.endm

SetWR:	.macro
	;; RF1 = !WR
		bset LATF, #1
	.endm 
ClrWR:	.macro
		bclr LATF, #1
	.endm

DisplayON:	.macro
	;; RF5 = DISPOFF active HIGH
		bset LATF, #5
	.endm
DisplayOFF:	.macro
		bclr LATF, #5
	.endm

ClrRes:	.macro
	;; RF4 = !RES
		bclr LATF, #4
	.endm
SetRes:	.macro
		bset LATF, #4
	.endm

Data:	.macro	data
		;; Data lines are PORTB<0:7>
		mov	\data, w0	
			;; When reffering to symbols within a macro, a "\" is used
		mov	w0, LATB
	.endm





;Initialize stack pointer and limit register
__reset: 
		mov #__SP_init, W15 ;Initialize the Stack Pointer register
		mov #__SPLIM_init, W0 ;Get address at the end of stack space
		mov W0, SPLIM ;Load the Stack Pointer Limit register
		nop ;Add NOP to follow SPLIM initialization
/*	
		; set PSVPAG to page that contains 'hello'
		mov #psvpage(Hello),w0
		mov w0,PSVPAG
		; enable Program Space Visibility
		bset.b CORCONL,#PSV
		; make a pointer to 'hello'
		mov #psvoffset(Hello),w8

		mov w8, LATB
*/
		
	
;; Initilization goes here

	;------------------------------------------------------------------------------
;Initialize LED outputs on PORTD bits 0-3
		call Delay2ms
		call InitializeSystem	;; Initialize dsPIC


;; Main loop 
Main:


	;; Idle the control lines and reset the display
	ClrRes
	SetWR
	SetRD
	ClrA0
	Data #0x00
	SetRes
	call Delay100ms

	Data #0x40
	call CommandData	;; System Set
	Data #0x38			;; Reg[00h]: Screen comp off, Dual Drive Panel,
	call WriteData		;;  8 pixel char height, internal CGROM
	Data #0x87			;; Reg[01h]: Two Frame AC Drive, Horiz Char Size (FX)
	call WriteData
	Data #0x07			;; Reg[02h]: Veritical Char Size (FY)
	call WriteData
	Data #0x3f			;; Reg[03h]: Char Bytes/Row (C/R)
	call WriteData
	Data #0x49			;; Reg[04h]: Total Char Bytes/Row (TC/R)
	call WriteData
	Data #0x7f			;; Reg[05h]: Frame Height 
	call WriteData
	Data #0x80			;; Reg[06h]: Horizontal Address Range 1
	call WriteData
	Data #0x00			;; Reg[07h]: Horizontal Address Range 2
	call WriteData


	Data #0x44			;; Scroll
	call CommandData
	Data #0x00			;; Reg[0Bh]: Screen Block Start Address 0
	call WriteData
	Data #0x00			;; Reg[0Ch]: Screen Block Start Address 1
	call WriteData
	Data #0x40			;; Reg[0Dh]: Screen Block 1 Size
	call WriteData
	Data #0x00			;; Reg[0Eh]: Screen Block 2 Start Address 0
	call WriteData
	Data #0x10			;; Reg[0Fh]: Screen Block 2 Start Address 1
	call WriteData
	Data #0x40			;; Reg[10h]: Screen Block 2 Size
	call WriteData
	Data #0x00			;; Reg[11h]: Screen Block 3 Start Address 0
	call WriteData
	Data #0x04			;; Reg[12h]: Screen Block 3 Start Address 1
	call WriteData
	Data #0x00			;; Reg[13h]: Screen Block 4 Start Address 0
	call WriteData
	Data #0x30 			;; Reg[13h]: Screen Block 4 Start Address 1
	call WriteData


	Data #0x5a			;; Horizontal Pixel Scroll
	call CommandData
	Data #0x00			;; Reg[1Bh]: 
	call WriteData


	Data #0x5b			;; Overlay
	call CommandData
	Data #0x01			;; Reg[18h]: Two layers, screen block 3-text mode
	call WriteData		;; Screen block 1-text mode, Exclusive-OR


	Data #0x58			;; Display Enable
	call CommandData	;; Reg[09h]: Diaplay Off
	Data #0x56			;; Reg[0Ah]: First/Second/Third/Fourth Screen Blocks on
	call WriteData		;; Cursor Flash at 2 Hz
	
	
	call EraseTextLayer	;; Clear data in first layer

	call EraseGraphicsLayer ;; Clear data in second layer


	Data #0x46			;; Cursor Write
	call CommandData
	Data #0x00			;; Reg[1Ch]: Cursor position at start of first screen
	call WriteData
	Data #0x00			;; Reg[1Dh]: 
	call WriteData


	Data #0x5d			;; Cursor Type
	call CommandData
	Data #0x04			;; Reg[15h]: Horizontal cursor size = 5 pixels
	call WriteData		;; NOTE on HSize ( Reg[15h]<3:0> <= Reg[01h]<3:0> )	
	Data #0x86			;; Reg[16h]: Block Cursor, Vertical Cursor size = 7 pixels
	call WriteData		;; NOTE on VSize ( Reg[16h]<3:0> <= Reg[02h]<3:0> )
	

	Data #0x59			;; Display Enable
	call CommandData    ;; Reg[09h]: Turn Display ON

	Data #0x4c			;; Cursor direction set to right
	call CommandData	
	
	Data #0x42			;; Write to LCD Memory
	call CommandData	
	Data #0x20			;; ''
	call WriteData
	Data #0x45			;; 'E'
	call WriteData
	Data #0x50			;; 'P'
	call WriteData
	Data #0x53			;; 'S'	
	call WriteData		
	Data #0x4f			;; 'O'
	call WriteData
	Data #0x4e			;; 'N'
	call WriteData
	
	;; Japanese Character Set
	Data #0x20			;; ' ' space
	call WriteData
	Data #0xc4			;; 'to' 
	call WriteData
	Data #0xda			;; 're'
	call WriteData
	Data #0xb3			;; 'v'
	call WriteData
	Data #0xb1			;; 'a'	
	call WriteData		
	Data #0xb0			;; extend character
	call WriteData
	


FlashLED:		;; Flash LED on PORTD to signify LCD setup complete
	btg LATD, #0
	call Delay

		bra FlashLED		;; Loop Back


;; End of the main loop 


;; Define Sub Routines
;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;; ******************************************************************************

;; LCD SUB-ROUTINES ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;; WRITE DATA ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;; Data macro must be used in conjunction with this to push the data being 
	;; written to the LCD onto PORTB
WriteData:  
		ClrA0		;; Clear A) line to write data
		ClrCS		;; Enable Epson Controller
		ClrWR		;; Strobe Write Line
		call Delay2ms
		SetWR
		SetCS		;; Disable controller
	return
;; END DATA WRITE ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;; COMMAND WRITE ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	;; Data macro must be used in conjunction with this to push the data being 
	;; written to the LCD onto PORTB
CommandData:
		SetA0		;; Set A0 line to write data
		ClrCS		;; Enable Epson controller
		ClrWR		;; Strobe WR line
		call Delay2ms	;; ideally the WAIT line should be polled
		SetWR
		SetCS		;; Disable controller
	return
;; END COMMAND WRITE ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


;; ERASE TEXT LAYER ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
EraseTextLayer:
		Data #0x46
		call CommandData
		Data #0x00
		call WriteData
		Data #0x00
		call WriteData
		
		Data #0x42
		call CommandData
		ClrA0
		ClrCS
		do	( #0x3A * 30 ), EndET
			Data #0x20 ;; Space Character
			ClrWR
			SetWR
EndET: 		nop
		SetCS
	return
;; END ERASE TEXT LAYER ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;; ERASE GRAPHICS LAYER ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
EraseGraphicsLayer:
		Data #0x46
		call CommandData
		Data #0x60
		call WriteData
		Data #0x09
		call WriteData
		
		Data #0x42
		call CommandData
		
		ClrA0
		ClrCS
		do ( #0x3A * 240 ), EndEG
			Data #0x00
			ClrWR
			SetWR
EndEG:		nop
		SetCS
	return

;; END ERASE GRAPHICS LAYER ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;; END LCD SUB-ROUTINES ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


;; FLASH LED STATUS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
LEDStatus: 
		bset LATD, #0 	;; Turn LED1 off
		bset LATD, #1	;; Turn LED2 off
		
		call Delay

		bclr LATD, #0	;; Turn LED1 on
		bset LATD, #1   ;; Turn LED2 off
		
		call Delay

		bset LATD, #0	;; Turn LED1 off
		bclr LATD, #1	;; Turn LED2 on
	
		call Delay
	
		bclr LATD, #0	;; Turn LED1 on
		bclr LATD, #1	;; Turn LED2 on

		call Delay
	return
;; END LED STATUS~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


;; INITIALIZE SYSTEM ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
InitializeSystem:

;; Port initializiation
;; Initialize LED outputs on PORTD bits 0-3
	;; Port pin driving the Anode of the LED
		mov #0xffff, W0 ;; Initialize LED pin data to off state
		mov W0, LATD	;; Move WZero into LATD
		nop				;; One instruction cycle is required between a port
						;; direction change or port write operation and a read
						;; operation of the same port.
		mov #0xfff0, W0 ;; Set LED pins as outputs
		mov W0, TRISD	;; Move WZero into LATD
		nop

		;; Turn on Power LED
		bclr	LATD, #0
		bset	LATD, #1

;; Initialize PORTF for control lines to LCD
	;; RF0 = !RD	<< Inputs at LCD ~ outputs at dsPIC
	;; RF1 = !WR    <<
	;; RF2 =  A0    <<
	;; RF3 = !CS    <<
	;; RF4 = !RES   <<
	;; RF5 = DISPOFF<<
	;; RF6 = WAIT	>> Output at LCD ~ input at dsPIC
		mov		#TRISF, w1	;; Pointer to TRISF
		mov		#0x0, 	w0	;; Clear PORTF
		mov		w0, LATF
		nop	
		mov		#0x40,  w0	;; Initialize PORTF<0:5> as outputs 
							;; and PORTF<5> as input
		mov.b	w0, [w1]	;; Move w0 to address in w1, TRISF
		nop
		
		
		;; Initialize PORTB as outputs for writing
		call PORTBWrite
	return ;; end InitializeSystem

;; Initialize PORTB for data lines to LCD
PORTBWrite:		;; Configure PORTB output for writing to LCD		
		mov   #TRISB, w1  ;; Pointer to TRISB
		mov   #0x0, w0 	  ;; Initialize PORTB as clear
		mov   w0, LATB
		nop
		mov   #0x00, w0	  ;; Initialize PORTB<0:7> as output
		mov.b w0, [w1]	  ;; Move w0 to TRISB via pointer
		nop				  ;; Equivalent to moving w0 to the address of w1/TRISB
	return

PORTBRead: ;; Configure PORTB as input for reading from LCD
		mov   #TRISB, w1	;; Pointer to TRISB
		mov   #0x0, w0   	;; Clear PORTB
		mov   w0, LATB
		nop
		mov   #0xff, w0		;; Initialize PORTB<0:7> as inputs
		mov.b w0, [w1]		
		nop

	return
			


		
	return



;; Initialize Clock Control Register
ClockSwitch:
		
	;; The following code sequence switch operation:
	;; Place the new oscillator in W0
	;; Place the new oscillator selection in W0
		mov #0x03, w0		;; Select external clock source OSCCON:NOSC<2:1>
		;OSCCONH (high byte) Unlock Sequence
		mov #OSCCONH, w1	;; Pointer to OSCCONH
		mov #0x78, w2		;; load first unlock code
		mov #0x9A, w3		;; Second 
		mov.b w2, [w1]		;; Write first
		mov.b w3, [w1]		;; Write second
		;Set new oscillator selection
		mov.b WREG, OSCCONH	;; Move w0 into OSCCON high byte
		;OSCCONL (low byte) unlock sequence
		mov #OSCCONL, w1	;; Pointer to OSCCONL
		mov.b #0x01, w0		;; Select OSCCON:OSWEN
		mov #0x46, w2		;; Load first unlock code
		mov #0x57, w3		;; Second
		mov.b w2, [w1]		;; Write first
		mov.b w3, [w1]		;; Write second
		;Start oscillator switch operation
		mov.b w0, [w1]		;; Set OSC switch enable bit

		;; Determine externally if OSC switch enable bit has been reset or not


	return

;; END INITIALIZE SYSTEM ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


;; DELAY SUB-ROUTINES ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Delay:	;; Based on 32.768 MHz clock input, Delay is currently 511.5 mSec
	
	mov	#0x3ff, W0		;; Move FFFF into WREG0
	mov W0, DelayCount	;; Move WREG0 into DelayCount
	mov W0, DelayCount2
	
LoopOutter:
	dec	DelayCount
	mov W0, DelayCount2
LoopA:
	dec DelayCount2
	cp0 DelayCount2
	bra NZ, LoopA
	cp0 DelayCount
	bra NZ, LoopOutter
	return


;; Delay for 100 miliseconds based on 32.768 MHz external clock input
Delay100ms:
	 do  #9990, Exit	;; Do outter loop 9990 times
		do #38, Inner	;; Do inner loop 38 times
		nop
Inner:  nop
	nop
Exit:nop
	return

Delay2ms:
	 do  #9990, D20ms	;; Do outter loop 9990 times
	 nop
D20ms:nop
	return


;; END DELAY ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;; End of sub Routine definition



;; Error Traps
; Default Exception Vector handlers if ALTIVT(INTCON2<15>) = 0
;------------------------------------------------------------------------------
; Oscillator Fail Trap
__OscillatorFail:
        bclr    INTCON1, #OSCFAIL
        nop		;; << Branches back to here
		;; turn on a LED on port A
        bra     $-2	;; Offset of the program counter, should be even number
        retfie

; Address Error Trap
__AddressError:
        bclr    INTCON1, #ADDRERR
        nop
        bra     $-2
        retfie

; Stack Error Trap
__StackError:
        bclr    INTCON1, #STKERR
        nop
        bra     $-2
        retfie

; Math Error Trap
__MathError:
        bclr  INTCON1, #MATHERR
        nop
        bra     $-2
        retfie


; Alternate Exception Vector handlers if ALTIVT(INTCON2<15>) = 1

; Alternate Oscillator Fail Trap
__AltOscillatorFail:
        bclr  INTCON1, #OSCFAIL
        nop
		nop
		nop	;; << jump abck here
		nop  
        bra     $-4
        retfie

; Alternate Address Error Trap
__AltAddressError:
        bclr  INTCON1, #ADDRERR
        nop
        bra     $-2
        retfie

; Alternate Stack Error Trap
__AltStackError:
        bclr  INTCON1, #STKERR
        nop
        bra     $-2
        retfie

; Alternate Math Error Trap
__AltMathError:
        bclr  INTCON1, #MATHERR
        nop
        bra     $-2
        retfie




	




;--------End of All Code Sections ---------------------------------------------

        .end                               ;End of program code in this file



